@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003-2013 GraphicsMagick Group
+% Copyright (C) 2003-2015 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -712,6 +712,24 @@ static void ZLIBFreeFunc(voidpf opaque, voidpf address)
 }
 #endif /* defined(HasZLIB) */
 
+#define ThrowMIFFReaderException(code_,reason_,image_) \
+do { \
+  if (number_of_profiles > 0) \
+    { \
+      unsigned int _index; \
+      for (_index=0; _index < number_of_profiles; _index++) \
+        { \
+          MagickFreeMemory(profiles[_index].name); \
+          MagickFreeMemory(profiles[_index].info); \
+        } \
+      MagickFreeMemory(profiles); \
+      number_of_profiles=0; \
+    } \
+  MagickFreeMemory(pixels); \
+  MagickFreeMemory(compress_pixels); \
+  ThrowReaderException(code_,reason_,image_); \
+} while (0);
+
 static Image *ReadMIFFImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -722,8 +740,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
 
   char
     id[MaxTextExtent],
-    keyword[MaxTextExtent],
-    *values;
+    keyword[MaxTextExtent];
 
   double
     version;
@@ -735,7 +752,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     c;
 
   size_t
-    length;
+    length,
+    compressed_length;
 
   long
     y;
@@ -753,8 +771,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     *p;
 
   unsigned char
-    *compress_pixels,
-    *pixels;
+    *compress_pixels = (unsigned char *) NULL,
+    *pixels = (unsigned char *) NULL;
 
   void
     *pixels_p;
@@ -806,10 +824,6 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     /*
       Decode image header;  header terminates one character beyond a ':'.
     */
-    length=MaxTextExtent;
-    values=MagickAllocateMemory(char *,length);
-    if (values == (char *) NULL)
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     colors=0;
     image->depth=8;
     image->compression=NoCompression;
@@ -824,22 +838,28 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
           char
             *comment;
 
+          size_t
+            comment_length;
+
           /*
             Read comment-- any text between { }.
           */
-          length=MaxTextExtent;
-          comment=MagickAllocateMemory(char *,length);
+          comment_length=MaxTextExtent;
+          comment=MagickAllocateMemory(char *,comment_length);
+          if (comment == (char *) NULL)
+            ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,
+              image);
           p=comment;
           for ( ; comment != (char *) NULL; p++)
           {
             c=ReadBlobByte(image);
             if ((c == EOF) || (c == '}'))
               break;
-            if ((size_t) (p-comment+1) >= length)
+            if ((size_t) (p-comment+1) >= comment_length)
               {
                 *p='\0';
-                length<<=1;
-                MagickReallocMemory(char *,comment,length);
+                comment_length<<=1;
+                MagickReallocMemory(char *,comment,comment_length);
                 if (comment == (char *) NULL)
                   break;
                 p=comment+strlen(comment);
@@ -847,7 +867,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             *p=c;
           }
           if (comment == (char *) NULL)
-            ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+            ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,
               image);
           *p='\0';
           (void) SetImageAttribute(image,"comment",comment);
@@ -857,39 +877,68 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
       else
         if (isalnum(c))
           {
+            char
+              *values;
+
+            size_t
+              values_length;
+
+            MagickBool
+              in_brace=MagickFalse;
+
             /*
-              Determine a keyword and its value.
+              Get keyword.
             */
+            keyword[0]='\0';
             p=keyword;
             do
             {
               if ((p-keyword) < (MaxTextExtent-1))
                 *p++=c;
               c=ReadBlobByte(image);
-            } while ((isalnum(c) || (c == '-') || (c == ':')) && (c != EOF));
+            } while ((c != '=') && (c != EOF));
             *p='\0';
-            while ((isspace(c) || (c == '=')) && (c != EOF))
+            if (c == EOF)
+              ThrowMIFFReaderException(CorruptImageWarning,ImproperImageHeader,image);
+
+            /*
+              Get values.
+
+              Values not containing spaces are terminated by the first
+              white-space (or new-line) enountered.  Values containing
+              spaces and/or new-lines must be surrounded by braces.
+            */
+            values_length=MaxTextExtent;
+            values=MagickAllocateMemory(char *,values_length);
+            if (values == (char *) NULL)
+              ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+            values[0]='\0';
+            c=ReadBlobByte(image);
+            in_brace=(c == '{');
+            if (in_brace)
               c=ReadBlobByte(image);
             p=values;
-            while ((c != '}') && (c != EOF))
-            {
-              if ((size_t) (p-values+1) >= length)
-                {
-                  *p='\0';
-                  length<<=1;
-                  MagickReallocMemory(char *,values,length);
-                  if (values == (char *) NULL)
+            while ((((!in_brace) && (c != '\n')) ||
+                     ((in_brace) && (c != '}'))) &&
+                   (c != EOF))
+              {
+                if ((size_t) (p-values+1) >= values_length)
+                  {
+                    *p='\0';
+                    values_length<<=1;
+                    MagickReallocMemory(char *,values,values_length);
+                    if (values == (char *) NULL)
+                      break;
+                    p=values+strlen(values);
+                  }
+                if (values == (char *) NULL)
+                  ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                *p++=c;
+                c=ReadBlobByte(image);
+                if (!in_brace)
+                  if (isspace(c))
                     break;
-                  p=values+strlen(values);
-                }
-              if (values == (char *) NULL)
-                ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-              *p++=c;
-              c=ReadBlobByte(image);
-              if (*values != '{')
-                if (isspace(c))
-                  break;
-            }
+              }
             *p='\0';
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                                   "keyword=\"%s\" values=\"%s\"",keyword,values);
@@ -1107,7 +1156,10 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                     i=(long) number_of_profiles;
                     MagickReallocMemory(ProfileInfo *,profiles,(i+1)*sizeof(ProfileInfo));
                     if (profiles == (ProfileInfo *) NULL)
-                      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                      {
+                        MagickFreeMemory(values);
+                        ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                      }
                     profiles[i].name=AllocateString(keyword+8);
                     profiles[i].length=MagickAtoL(values);
                     profiles[i].info=(unsigned char *) NULL;
@@ -1221,13 +1273,15 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                 break;
               }
             }
+            MagickFreeMemory(values);
           }
         else
-          c=ReadBlobByte(image);
+          {
+            c=ReadBlobByte(image);
+          }
       while (isspace(c))
         c=ReadBlobByte(image);
     }
-    MagickFreeMemory(values);
     (void) ReadBlobByte(image);
 
     (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -1248,14 +1302,13 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
       {
         if (image->previous)
           {
-            /* Recover from failure to read image in sequence */
-            (void) RemoveLastImageFromList(&image);
+            DeleteImageFromList(&image);
             ThrowException(exception,CorruptImageWarning,ImproperImageHeader,image->filename);
             break;
           }
         else
           {
-            ThrowReaderException(CorruptImageError,ImproperImageHeader,
+            ThrowMIFFReaderException(CorruptImageError,ImproperImageHeader,
               image);
           }
       }
@@ -1280,7 +1333,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         */
         image->directory=MagickAllocateMemory(char *,MaxTextExtent);
         if (image->directory == (char *) NULL)
-          ThrowReaderException(CorruptImageError,UnableToReadImageData,image);
+          ThrowMIFFReaderException(CorruptImageError,UnableToReadImageData,image);
         p=image->directory;
         do
         {
@@ -1293,7 +1346,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               MagickReallocMemory(char *,image->directory,
                 (strlen(image->directory)+MaxTextExtent+1));
               if (image->directory == (char *) NULL)
-                ThrowReaderException(CorruptImageError,UnableToReadImageData,
+                ThrowMIFFReaderException(CorruptImageError,UnableToReadImageData,
                   image);
               p=image->directory+strlen(image->directory);
             }
@@ -1313,7 +1366,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             continue;
           profiles[i].info=MagickAllocateMemory(unsigned char *,profiles[i].length);
           if (profiles[i].info == (unsigned char *) NULL)
-            ThrowReaderException(CorruptImageError,UnableToReadGenericProfile,
+            ThrowMIFFReaderException(CorruptImageError,UnableToReadGenericProfile,
               image);
           (void) ReadBlob(image,profiles[i].length,profiles[i].info);
           (void) SetImageProfile(image,profiles[i].name,profiles[i].info,profiles[i].length);
@@ -1330,7 +1383,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
           Create image colormap.
         */
         if (!AllocateImageColormap(image,colors != 0 ? colors : 256))
-          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+          ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
         if (colors != 0)
           {
@@ -1347,7 +1400,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             packet_size=3*depth/8;
             colormap=MagickAllocateMemory(unsigned char *,packet_size*image->colors);
             if (colormap == (unsigned char *) NULL)
-              ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+              ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,
                 image);
             (void) ReadBlob(image,packet_size*image->colors,colormap);
             p=colormap;
@@ -1403,7 +1456,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         break;
 
     if (CheckImagePixelLimits(image, exception) != MagickPass)
-      ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+      ThrowMIFFReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
 
     /*
       Determine import properties
@@ -1455,12 +1508,16 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
       packet_size+=quantum_size/8;
     if (image->compression == RLECompression)
       packet_size++;
-    pixels=MagickAllocateMemory(unsigned char *,packet_size*image->columns);
-    length=(size_t) (1.01*packet_size*image->columns+600);
-    compress_pixels=MagickAllocateMemory(unsigned char *,length);
-    if ((pixels == (unsigned char *) NULL) ||
-        (compress_pixels == (unsigned char *) NULL))
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    pixels=MagickAllocateArray(unsigned char *,packet_size,image->columns);
+    if (pixels == (unsigned char *) NULL)
+      ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    length=(size_t) (1.01*MagickArraySize(packet_size,image->columns));
+    if (length)
+      length += 600;
+    compressed_length = length;
+    compress_pixels=MagickAllocateMemory(unsigned char *,compressed_length);
+    if (compress_pixels == (unsigned char *) NULL)
+      ThrowMIFFReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     /*
       Read image pixels.
     */
@@ -1497,19 +1554,38 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   if (zip_info.avail_in == 0)
                     {
                       zip_info.next_in=compress_pixels;
-                      length=(int) (1.01*packet_size*image->columns+12);
-                      if (version != 0)
-                        length=ReadBlobMSBLong(image);
-                      zip_info.avail_in=(uInt) ReadBlob(image,length,zip_info.next_in);
+                      if (version == 0)
+                        {
+                          length=(int) (1.01*packet_size*image->columns+12);
+                          zip_info.avail_in=(uInt) ReadBlob(image,length,zip_info.next_in);
+                        }
+                      else
+                        {
+                          length=ReadBlobMSBLong(image);
+                          if (length > compressed_length)
+                            {
+                              (void) inflateEnd(&zip_info);
+                              ThrowMIFFReaderException(CorruptImageError,
+                                                       LengthAndFilesizeDoNotMatch,
+                                                       image);
+                            }
+                          zip_info.avail_in=(uInt) ReadBlob(image,length,zip_info.next_in);
+                          if ((size_t) zip_info.avail_in != length)
+                            {
+                              (void) inflateEnd(&zip_info);
+                              ThrowMIFFReaderException(CorruptImageError,
+                                                       UnexpectedEndOfFile,
+                                                       image);
+                            }
+                        }
                     }
                   zip_status=inflate(&zip_info,Z_NO_FLUSH);
                   if (zip_status == Z_STREAM_END)
                     break;
                   else if (zip_status != Z_OK)
                     {
-                      MagickFreeMemory(pixels);
-                      MagickFreeMemory(compress_pixels);
-                      ThrowReaderException(CorruptImageError,UnableToUncompressImage,
+                      (void) inflateEnd(&zip_info);
+                      ThrowMIFFReaderException(CorruptImageError,UnableToUncompressImage,
                                            image);
                     }
                 } while (zip_info.avail_out != 0);
@@ -1564,19 +1640,29 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   if (bzip_info.avail_in == 0)
                     {
                       bzip_info.next_in=(char *) compress_pixels;
-                      length=(int) (1.01*packet_size*image->columns+600);
-                      if (version != 0)
-                        length=ReadBlobMSBLong(image);
-                      bzip_info.avail_in=(unsigned int) ReadBlob(image,length,bzip_info.next_in);
+                      if (version == 0)
+                        {
+                          length=(int) (1.01*packet_size*image->columns+600);
+                          bzip_info.avail_in=(unsigned int) ReadBlob(image,length,bzip_info.next_in);
+                        }
+                      else
+                        {
+                          length=ReadBlobMSBLong(image);
+                          bzip_info.avail_in=(unsigned int) ReadBlob(image,length,bzip_info.next_in);
+                          if ((size_t) bzip_info.avail_in != length)
+                            {
+                              ThrowMIFFReaderException(CorruptImageError,UnexpectedEndOfFile,
+                                                   image);
+                            }
+                        }
                     }
                   bz_status=BZ2_bzDecompress(&bzip_info);
                   if (bz_status == BZ_STREAM_END)
                     break;
                   else if (bz_status != BZ_OK)
                     {
-                      MagickFreeMemory(pixels);
-                      MagickFreeMemory(compress_pixels);
-                      ThrowReaderException(CorruptImageError,UnableToUncompressImage,
+                      (void) BZ2_bzDecompressEnd(&bzip_info);
+                      ThrowMIFFReaderException(CorruptImageError,UnableToUncompressImage,
                                            image);
                     }
                 } while (bzip_info.avail_out != 0);
@@ -1849,14 +1935,20 @@ static void WriteRunlengthPacket(const Image *image,
           {
             *q++=(unsigned char) (value >> 24);
             *q++=(unsigned char) (value >> 16);
+            *q++=(unsigned char) (value >> 8);
+            *q++=(unsigned char) value;
+            break;
           }
         case 16:
           {
             *q++=(unsigned char) (value >> 8);
+            *q++=(unsigned char) value;
+            break;
           }
         case 8:
           {
             *q++=(unsigned char) value;
+            break;
           }
         } /* switch (quantum_size) */
       if (image->matte)
@@ -1868,14 +1960,20 @@ static void WriteRunlengthPacket(const Image *image,
               {
                 *q++=(unsigned char) (value >> 24);
                 *q++=(unsigned char) (value >> 16);
+                *q++=(unsigned char) (value >> 8);
+                *q++=(unsigned char) value;
+                break;
               }
             case 16:
               {
                 *q++=(unsigned char) (value >> 8);
+                *q++=(unsigned char) value;
+                break;
               }
             case 8:
               {
                 *q++=(unsigned char) value;
+                break;
               }
             } /* switch (quantum_size) */
         }
@@ -1981,6 +2079,13 @@ static void WriteRunlengthPacket(const Image *image,
   *qp=q;
 }
 
+#define ThrowMIFFWriterException(code_,reason_,image_) \
+{ \
+  MagickFreeMemory(compress_pixels)            \
+  MagickFreeMemory(pixels); \
+  ThrowWriterException(code_,reason_,image_); \
+}
+
 static unsigned int WriteMIFFImage(const ImageInfo *image_info,Image *image)
 {
 
@@ -2027,8 +2132,8 @@ static unsigned int WriteMIFFImage(const ImageInfo *image_info,Image *image)
     length;
 
   unsigned char
-    *compress_pixels,
-    *pixels;
+    *compress_pixels = (unsigned char *) NULL,
+    *pixels = (unsigned char *) NULL;
 
   unsigned int
     depth,
@@ -2124,7 +2229,7 @@ static unsigned int WriteMIFFImage(const ImageInfo *image_info,Image *image)
     compress_pixels=MagickAllocateMemory(unsigned char *,length);
     if ((pixels == (unsigned char *) NULL) ||
         (compress_pixels == (unsigned char *) NULL))
-      ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+      ThrowMIFFWriterException(ResourceLimitError,MemoryAllocationFailed,image);
     /*
       Write MIFF header.
     */
@@ -2383,8 +2488,8 @@ static unsigned int WriteMIFFImage(const ImageInfo *image_info,Image *image)
         packet_size=3*depth/8;
         colormap=MagickAllocateMemory(unsigned char *,packet_size*image->colors);
         if (colormap == (unsigned char *) NULL)
-          ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,
-            image);
+          ThrowMIFFWriterException(ResourceLimitError,MemoryAllocationFailed,
+                                   image);
         /*
           Write colormap to file.
         */
