@@ -66,25 +66,26 @@
   pixmap.plane_bytes=ReadBlobMSBLong(image);                           \
   pixmap.table=ReadBlobMSBLong(image);                                 \
   pixmap.reserved=ReadBlobMSBLong(image);                              \
-  if (EOFBlob(image) ||                                                \
-      pixmap.bits_per_pixel <= 0 || pixmap.bits_per_pixel > 32 ||      \
-      pixmap.component_count <= 0 || pixmap.component_count > 4 ||     \
-      pixmap.component_size <= 0)                                      \
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image); \
   }
 
+#define ValidatePixmap(pixmap) \
+  (!(EOFBlob(image) ||                                                 \
+     pixmap.bits_per_pixel <= 0 || pixmap.bits_per_pixel > 32 ||       \
+     pixmap.component_count <= 0 || pixmap.component_count > 4 ||      \
+     pixmap.component_size <= 0))
+
 #define ReadRectangle(rectangle)                                       \
   {                                                                    \
   rectangle.top=ReadBlobMSBShort(image);                               \
   rectangle.left=ReadBlobMSBShort(image);                              \
   rectangle.bottom=ReadBlobMSBShort(image);                            \
   rectangle.right=ReadBlobMSBShort(image);                             \
-  if (EOFBlob(image) ||                                                \
-      (rectangle.top > rectangle.bottom ||                             \
-       rectangle.left > rectangle.right))                              \
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image); \
   }
 
+#define ValidateRectangle(rectangle) \
+  (!(EOFBlob(image) || \
+     (rectangle.top > rectangle.bottom || rectangle.left > rectangle.right)))
+
 typedef struct _PICTCode
 {
   char
@@ -769,6 +770,13 @@ static size_t EncodeImage(Image *image,const unsigned char *scanline,
 %
 %
 */
+#define ThrowPICTReaderException(code_,reason_,image_) \
+{ \
+  if (clone_info) \
+    DestroyImageInfo(clone_info); \
+  ThrowReaderException(code_,reason_,image_); \
+}
+
 static Image *ReadPICTImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
@@ -778,6 +786,9 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   Image
     *image;
 
+  ImageInfo
+    *clone_info = (ImageInfo *) NULL;
+
   IndexPacket
     index;
 
@@ -826,7 +837,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == False)
-    ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+    ThrowPICTReaderException(FileOpenError,UnableToOpenFile,image);
   pixmap.bits_per_pixel=0;
   pixmap.component_count=0;
   /*
@@ -838,26 +849,28 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
     (void) ReadBlobByte(image);  /* skip header */
   (void) ReadBlobMSBShort(image);  /* skip picture size */
   ReadRectangle(frame);
+  if (!ValidateRectangle(frame))
+    ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
   while ((c=ReadBlobByte(image)) == 0);
   if (c != 0x11)
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+    ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
   version=ReadBlobByte(image);
   if (version == 2)
     {
       c=ReadBlobByte(image);
       if (c != 0xff)
-        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+        ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
     }
   else
     if (version != 1)
-      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
   /*
     Validate dimensions
   */
   if ((frame.left < 0) || (frame.right < 0) || (frame.top < 0) || (frame.bottom < 0) ||
       (frame.left >= frame.right) || (frame.top >= frame.bottom))
     {
-      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
     }
 
   /*
@@ -872,7 +885,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                           "Dimensions: %lux%lu",image->columns,image->rows);
 
   if (CheckImagePixelLimits(image, exception) != MagickPass)
-    ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+    ThrowPICTReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
 
   /*
     Interpret PICT opcodes.
@@ -912,12 +925,14 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             if ((frame.left & 0x8000) || (frame.top & 0x8000))
               break;
             image->columns=frame.right-frame.left;
             image->rows=frame.bottom-frame.top;
             if (CheckImagePixelLimits(image, exception) != MagickPass)
-              ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+              ThrowPICTReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
             (void) SetImageEx(image,OpaqueOpacity,exception);
             break;
           }
@@ -945,11 +960,15 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 break;
               }
             if (pattern != 1)
-              ThrowReaderException(CorruptImageError,UnknownPatternType,
+              ThrowPICTReaderException(CorruptImageError,UnknownPatternType,
                 image);
             length=ReadBlobMSBShort(image);
             ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             ReadPixmap(pixmap);
+            if (!ValidatePixmap(pixmap))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             (void) ReadBlobMSBLong(image);
             flags=ReadBlobMSBShort(image);
             length=ReadBlobMSBShort(image);
@@ -1050,6 +1069,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 (void) ReadBlobMSBShort(image);
               }
             ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             /*
               Initialize tile image.
             */
@@ -1062,6 +1083,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             if ((code == 0x9a) || (code == 0x9b) || (bytes_per_line & 0x8000))
               {
                 ReadPixmap(pixmap);
+                if (!ValidatePixmap(pixmap))
+                  ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
                 tile_image->matte=pixmap.component_count == 4;
               }
             if ((code != 0x9a) && (code != 0x9b))
@@ -1079,7 +1102,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (!AllocateImageColormap(tile_image,tile_image->colors))
                   {
                     DestroyImage(tile_image);
-                    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
+                    ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image)
                   }
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
                   "Allocated tile image colormap with %u colors",tile_image->colors);
@@ -1112,7 +1135,11 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                   }
               }
             ReadRectangle(source);
+            if (!ValidateRectangle(source))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             ReadRectangle(destination);
+            if (!ValidateRectangle(destination))
+              ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
             (void) ReadBlobMSBShort(image);
             if ((code == 0x91) || (code == 0x99) || (code == 0x9b))
               {
@@ -1133,7 +1160,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
               {
                 CopyException(exception, &tile_image->exception);
                 DestroyImage(tile_image);
-                ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
+                ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image)
               }
             /*
               Convert PICT tile image to pixel packets.
@@ -1244,7 +1271,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (length == 0)
                   break;
                 if (SetImageProfile(image,"ICM",info,length) == MagickFail)
-                  ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                  ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
                 MagickFreeMemory(info);
                 break;
               }
@@ -1253,7 +1280,7 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
                 if (length == 0)
                   break;
                 if (SetImageProfile(image,"IPTC",info,length) == MagickFail)
-                  ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                  ThrowPICTReaderException(ResourceLimitError,MemoryAllocationFailed,image);
                 MagickFreeMemory(info);
                 break;
               }
@@ -1296,9 +1323,6 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         Image
           *tile_image;
 
-        ImageInfo
-          *clone_info;
-
         /*
           Embedded JPEG.
         */
@@ -1313,16 +1337,26 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
             ThrowReaderTemporaryFileException(clone_info->filename);
           }
         length=ReadBlobMSBLong(image);
-        for (i=0; i < 6; i++)
-          (void) ReadBlobMSBLong(image);
-        ReadRectangle(frame);
-        for (i=0; i < 122; i++)
-          (void) ReadBlobByte(image);
-        for (i=0; i < (long) (length-154); i++)
-        {
-          c=ReadBlobByte(image);
-          (void) fputc(c,file);
-        }
+        if (length > 154)
+          {
+            for (i=0; i < 6; i++)
+              (void) ReadBlobMSBLong(image);
+            ReadRectangle(frame);
+            if (!ValidateRectangle(frame))
+              {
+                (void) fclose(file);
+                ThrowPICTReaderException(CorruptImageError,ImproperImageHeader,image);
+              }
+            for (i=0; i < 122; i++)
+              if (ReadBlobByte(image) == EOF)
+                break;
+            for (i=0; i < (long) (length-154); i++)
+              {
+                if ((c=ReadBlobByte(image)) == EOF)
+                  break;
+                (void) fputc(c,file);
+              }
+          }
         (void) fclose(file);
         tile_image=ReadImage(clone_info,exception);
         (void) LiberateTemporaryFile(clone_info->filename);
@@ -1348,7 +1382,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         */
         length=ReadBlobMSBShort(image);
         for (i=0; i < (long) length; i++)
-          (void) ReadBlobByte(image);
+          if (ReadBlobByte(image) == EOF)
+            break;
         continue;
       }
     if ((code >= 0x100) && (code <= 0x7fff))
@@ -1358,7 +1393,8 @@ static Image *ReadPICTImage(const ImageInfo *image_info,
         */
         length=(code >> 7) & 0xff;
         for (i=0; i < (long) length; i++)
-          (void) ReadBlobByte(image);
+          if (ReadBlobByte(image) == EOF)
+            break;
         continue;
       }
   }
@@ -1471,6 +1507,17 @@ ModuleExport void UnregisterPICTImage(void)
 %
 %
 */
+#define LiberatePICTAllocations()               \
+  {                                             \
+    MagickFreeMemory(buffer);                   \
+    MagickFreeMemory(packed_scanline);          \
+    MagickFreeMemory(scanline);                 \
+  }
+#define ThrowPICTWriterException(code_,reason_,image_)  \
+  {                                                     \
+    LiberatePICTAllocations();                          \
+    ThrowWriterException(code_,reason_,image_);         \
+  }
 static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
 {
 #define MaxCount  128U
@@ -1524,9 +1571,9 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
     count;
 
   unsigned char
-    *buffer,
-    *packed_scanline,
-    *scanline;
+    *buffer = (unsigned char *) NULL,
+    *packed_scanline = (unsigned char *) NULL,
+    *scanline = (unsigned char *) NULL;
 
   unsigned int
     status;
@@ -1548,10 +1595,10 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if ((image->columns > 65535L) || (image->rows > 65535L))
-    ThrowWriterException(ImageError,WidthOrHeightExceedsLimit,image);
+    ThrowPICTWriterException(ImageError,WidthOrHeightExceedsLimit,image);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == False)
-    ThrowWriterException(FileOpenError,UnableToOpenFile,image);
+    ThrowPICTWriterException(FileOpenError,UnableToOpenFile,image);
   (void) TransformColorspace(image,RGBColorspace);
   /*
     Initialize image info.
@@ -1607,7 +1654,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
   if ((buffer == (unsigned char *) NULL) ||
       (packed_scanline == (unsigned char *) NULL) ||
       (scanline == (unsigned char *) NULL))
-    ThrowWriterException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowPICTWriterException(ResourceLimitError,MemoryAllocationFailed,image);
   /*
     Write header, header size, size bounding box, version, and reserved.
   */
@@ -1687,6 +1734,7 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
       jpeg_image=CloneImage(image,0,0,True,&image->exception);
       if (jpeg_image == (Image *) NULL)
         {
+          LiberatePICTAllocations();
           CloseBlob(image);
           return (False);
         }
@@ -1697,7 +1745,11 @@ static unsigned int WritePICTImage(const ImageInfo *image_info,Image *image)
         &image->exception);
       DestroyImage(jpeg_image);
       if (blob == (unsigned char *) NULL)
-        return(False);
+        {
+          LiberatePICTAllocations();
+          CloseBlob(image);
+          return(False);
+        }
       (void) WriteBlobMSBShort(image,PictJPEGOp);
       (void) WriteBlobMSBLong(image,(const magick_uint16_t) length+154);
       (void) WriteBlobMSBShort(image,0x0000);
