@@ -528,6 +528,17 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 	    }
 	}
 
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Dimensions: %lux%lu",
+                            image->columns,image->rows);
+      if ((image->columns == 0) || (image->rows == 0))
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Max Value: %u",
+			    max_value);
+      if ((max_value == 0) || (max_value > 4294967295U))
+        ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+
+      bits_per_sample=0;
       if (max_value <= 1)
 	bits_per_sample=1;
       else if (max_value <= 255U)
@@ -539,10 +550,6 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
       image->depth=Min(bits_per_sample,QuantumDepth);
 
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Max Value: %u",
-			    max_value);
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Dimensions: %lux%lu",
-                            image->columns,image->rows);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Image Depth: %u",
                             image->depth);
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Samples Per Pixel: %u",
@@ -596,6 +603,10 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       if (image_info->ping && (image_info->subrange != 0))
         if (image->scene >= (image_info->subimage+image_info->subrange-1))
           break;
+
+      if (CheckImagePixelLimits(image, exception) != MagickPass)
+        ThrowReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+
       /*
         Convert PNM pixels to runlength-encoded MIFF packets.
       */
@@ -2023,10 +2034,11 @@ static unsigned int WritePNMImage(const ImageInfo *image_info,Image *image)
       if (image->next == (Image *) NULL)
 	break;
       image=SyncNextImageInList(image);
-      status=MagickMonitorFormatted(scene++,GetImageListLength(image),
-				    &image->exception,SaveImagesText,
-				    image->filename);
-      if (status == False)
+      if (status != MagickFail)
+        status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                      &image->exception,SaveImagesText,
+                                      image->filename);
+      if (status == MagickFail)
 	break;
     } while (image_info->adjoin);
   if (image_info->adjoin)
