@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003 - 2015 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -41,6 +41,7 @@
 #include "magick/color.h"
 #include "magick/color_lookup.h"
 #include "magick/colormap.h"
+#include "magick/log.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
@@ -131,7 +132,7 @@ static char *ParseColor(char *data)
 #define NumberTargets  6
 
   static const char
-    *targets[NumberTargets] = { "c ", "g ", "g4 ", "m ", "b ", "s " };
+    * const targets[NumberTargets] = { "c ", "g ", "g4 ", "m ", "b ", "s " };
 
   register char
     *p,
@@ -165,15 +166,28 @@ static char *ParseColor(char *data)
   }
   return((char *) NULL);
 }
+#define ThrowXPMReaderException(code_,reason_,image_) \
+do { \
+  if (keys) \
+    for (i=0; i < (long) image->colors; i++) \
+      MagickFreeMemory(keys[i]); \
+  MagickFreeMemory(keys); \
+  if (textlist) \
+    for (i=0; textlist[i] != (char *) NULL; i++) \
+      MagickFreeMemory(textlist[i]); \
+  MagickFreeMemory(textlist); \
+  MagickFreeMemory(xpm_buffer); \
+  ThrowReaderException(code_,reason_,image_); \
+} while (0);
 
 static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
     key[MaxTextExtent],
-    **keys,
+    **keys = (char **) NULL,
     target[MaxTextExtent],
-    **textlist,
-    *xpm_buffer;
+    **textlist = (char **) NULL,
+    *xpm_buffer = (char *) NULL;
 
   Image
     *image;
@@ -183,6 +197,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   long
     j,
+    k,
     none,
     y;
 
@@ -218,33 +233,36 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
-  if (status == False)
-    ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+  if (status == MagickFail)
+    ThrowXPMReaderException(FileOpenError,UnableToOpenFile,image);
   /*
     Read XPM file.
   */
   length=MaxTextExtent;
   xpm_buffer=MagickAllocateMemory(char *,length);
-  p=xpm_buffer;
   if (xpm_buffer != (char *) NULL)
-    while (ReadBlobString(image,p) != (char *) NULL)
     {
-      if (*p == '#')
-        if ((p == xpm_buffer) || (*(p-1) == '\n'))
-          continue;
-      if ((*p == '}') && (*(p+1) == ';'))
-        break;
-      p+=strlen(p);
-      if ((p-xpm_buffer+MaxTextExtent+1) < (long) length)
-        continue;
-      length<<=1;
-      MagickReallocMemory(char *,xpm_buffer,length);
-      if (xpm_buffer == (char *) NULL)
-        break;
-      p=xpm_buffer+strlen(xpm_buffer);
+      xpm_buffer[0]='\0';
+      p=xpm_buffer;
+      while (ReadBlobString(image,p) != (char *) NULL)
+        {
+          if (*p == '#')
+            if ((p == xpm_buffer) || (*(p-1) == '\n'))
+              continue;
+          if ((*p == '}') && (*(p+1) == ';'))
+            break;
+          p+=strlen(p);
+          if ((p-xpm_buffer+MaxTextExtent+1) < (long) length)
+            continue;
+          length<<=1;
+          MagickReallocMemory(char *,xpm_buffer,length);
+          if (xpm_buffer == (char *) NULL)
+            break;
+          p=xpm_buffer+strlen(xpm_buffer);
+        }
     }
   if (xpm_buffer == (char *) NULL)
-    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowXPMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   /*
     Remove comments.
   */
@@ -260,7 +278,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   }
   if ((count != 4) || (width > 2) || (image->columns == 0) ||
       (image->rows == 0) || (image->colors == 0))
-    ThrowReaderException(CorruptImageError,ImproperImageHeader,image)
+    ThrowXPMReaderException(CorruptImageError,ImproperImageHeader,image);
   image->depth=16;
   /*
     Remove unquoted characters.
@@ -273,7 +291,13 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
     for (q=p+1; *q != '\0'; q++)
       if (*q == '"')
         break;
-    (void) strncpy(xpm_buffer+i,p+1,q-p-1);
+    /* loop below equivalent to (void) strncpy(xpm_buffer+i,p+1,q-p-1); */
+    for ( k=0; k < (q-p-1); k++ )
+      {
+        (xpm_buffer+i)[k]=(p+1)[k];
+        if ((p+1)[k] == '\0')
+          break;
+      }
     i+=q-p-1;
     xpm_buffer[i++]='\n';
     p=q+1;
@@ -282,18 +306,25 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   textlist=StringToList(xpm_buffer);
   MagickFreeMemory(xpm_buffer);
   if (textlist == (char **) NULL)
-    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowXPMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+  if (image->logging)
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                            "TextList");
+      for (i=0; textlist[i] != (char *) NULL; i++)
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                              "    %lu: %s", i, textlist[i]);
+    }
+  
   /*
     Initialize image structure.
   */
-  keys=MagickAllocateMemory(char **,image->colors*sizeof(char *));
+  keys=MagickAllocateArray(char **,image->colors,sizeof(char *));
   if (!AllocateImageColormap(image,image->colors) || (keys == (char **) NULL))
-    {
-      for (i=0; textlist[i] != (char *) NULL; i++)
-        MagickFreeMemory(textlist[i]);
-      MagickFreeMemory(textlist);
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
-    }
+    ThrowXPMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+  for (i=0; i < (long) image->colors; i++)
+    keys[i]=(char *) NULL;
+
   /*
     Read image colormap.
   */
@@ -306,13 +337,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     keys[j]=MagickAllocateMemory(char *,width+1);
     if (keys[j] == (char *) NULL)
-      {
-        for (i=0; textlist[i] != (char *) NULL; i++)
-          MagickFreeMemory(textlist[i]);
-        MagickFreeMemory(textlist);
-        MagickFreeMemory(keys);
-        ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image)
-      }
+      ThrowXPMReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     keys[j][width]='\0';
     (void) strncpy(keys[j],p,width);
     /*
@@ -341,12 +366,7 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
   }
   if (j < (long) image->colors)
-    {
-      for (i=0; textlist[i] != (char *) NULL; i++)
-        MagickFreeMemory(textlist[i]);
-      MagickFreeMemory(textlist);
-      ThrowReaderException(CorruptImageError,CorruptImage,image)
-    }
+    ThrowXPMReaderException(CorruptImageError,CorruptImage,image);
   image->depth=GetImageDepth(image,&image->exception);
   image->depth=NormalizeDepthToOctet(image->depth);
   j=0;
@@ -361,16 +381,25 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
         p=textlist[i++];
         if (p == (char *) NULL)
           break;
-        r=SetImagePixels(image,0,y,image->columns,1);
+        r=SetImagePixelsEx(image,0,y,image->columns,1,exception);
         if (r == (PixelPacket *) NULL)
-          {
-            CopyException(exception,&image->exception);
-            break;
-          }
+          break;
         indexes=AccessMutableIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          (void) strncpy(key,p,width);
+          /* (void) strncpy(key,p,width); */
+          for (k=0; k < (long) width; k++)
+            {
+              key[k]=p[k];
+              if (p[k] == '\0')
+                {
+                  status=MagickFail;
+                  break;
+                }
+            }
+          if (MagickFail == status)
+            break;
+          key[k]='\0';
           if (strcmp(key,keys[j]) != 0)
             for (j=0; j < (long) Max(image->colors-1,1); j++)
               if (strcmp(key,keys[j]) == 0)
@@ -383,14 +412,13 @@ static Image *ReadXPMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           r++;
           p+=width;
         }
-        if (!SyncImagePixels(image))
-          {
-            CopyException(exception,&image->exception);
-            break;
-          }
+        if (MagickFail == status)
+          break;
+        if (!SyncImagePixelsEx(image,exception))
+          break;
       }
       if (y < (long) image->rows)
-        ThrowReaderException(CorruptImageError,InsufficientImageDataInFile,image);
+        ThrowXPMReaderException(CorruptImageError,InsufficientImageDataInFile,image);
     }
   /*
     Free resources.
@@ -522,7 +550,7 @@ static unsigned int WritePICONImage(const ImageInfo *image_info,Image *image)
 #define GraymapExtent  95
 #define PiconGeometry  "48x48>"
 
-  static unsigned char
+  static const unsigned char
     Colormap[]=
     {
       0x47, 0x49, 0x46, 0x38, 0x37, 0x61, 0x06, 0x00, 0x05, 0x00, 0xf4, 0x05,
